'''
Created on 2015/6/29

:author: hubo
'''
from __future__ import print_function
import unittest
from vlcp.event.event import Event, withIndices
from vlcp.event.core import Scheduler, PollEvent, TimerEvent, SystemControlEvent, SystemControlLowPriorityEvent
from vlcp.event import DefaultPolling
from vlcp.event.connection import Client, TcpServer, ConnectionWriteEvent, ConnectionControlEvent,\
    ResolveRequestEvent, ResolveResponseEvent
from vlcp.utils.connector import Resolver
from vlcp.protocol.protocol import Protocol
from vlcp.event.runnable import RoutineContainer
from socket import SOL_SOCKET, SO_ERROR
from vlcp.event.pqueue import CBQueue
from vlcp.event.runnable import RoutineControlEvent
import errno
import logging
import socket
import os

@withIndices('connection')
class TestDataEvent(Event):
    canignore = False

@withIndices()
class TestWriteEvent(ConnectionWriteEvent):
    pass

class TestProtocol(Protocol):
    _logger = logging.getLogger('TestProtocol')
    class ResponderRoutine(RoutineContainer):
        def __init__(self, connection, passive):
            scheduler = connection.scheduler
            RoutineContainer.__init__(self, scheduler)
            self.passive = passive
            self.connection = connection
        def main(self):
            datamatcher = TestDataEvent.createMatcher(connection=self.connection)
            if self.passive:
                yield (datamatcher,)
                self.event.canignore = True
            for i in range(0,9):
                event = TestWriteEvent(self.connection, self.connection.connmark)
                for m in self.waitForSend(event):
                    yield m
                yield (datamatcher,)
                self.event.canignore = True
            event = TestWriteEvent(self.connection, self.connection.connmark)
            for m in self.waitForSend(event):
                yield m
            if not self.passive:
                yield (datamatcher,)
                self.event.canignore = True
            for m in self.connection.shutdown():
                yield m
    def __init__(self, passive):
        Protocol.__init__(self)
        self.passive = passive
    def parse(self, connection, data, laststart):
        l = len(data)
        events = []
        for i in range(0, l//1000):
            events.append(TestDataEvent(connection))
        return (events, l % 1000)
    def serialize(self, connection, event):
        return (b'a' * 1000, False)
    def init(self, connection):
        for m in Protocol.init(self, connection):
            yield m
        rr = self.ResponderRoutine(connection, self.passive)
        connection.responder = rr
        rr.start(False)
        if False:
            yield
    def error(self, connection):
        for m in Protocol.error(self, connection):
            yield m
        err = connection.socket.getsockopt(SOL_SOCKET, SO_ERROR)
        self._logger.warning('Connection error status: %d(%s)', err, errno.errorcode.get(err, 'Not found'))
        connection.responder.mainroutine.close()
        if False:
            yield
    def notconnected(self, connection):
        self._logger.warning('Connect failed and not retrying for url: %s', connection.rawurl)
        if False:
            yield
    def closed(self, connection):
        for m in Protocol.closed(self, connection):
            yield m
        connection.responder.mainroutine.close()
        if False:
            yield
    def accept(self, server, newaddr, newsocket):
        self._logger.debug('Connection accepted from ' + repr(newaddr))
        return Protocol.accept(self, server, newaddr, newsocket)

class TestConnection(unittest.TestCase):


    def setUp(self):
        #self.scheduler 
        logging.basicConfig()
        self.scheduler = Scheduler(DefaultPolling())
        #self.scheduler.debugging = True
        #self.scheduler.logger.setLevel('DEBUG')
        #Client.logger.setLevel('DEBUG')
        import tests
        import os.path
        rootpath, _ = os.path.split(tests.__path__[0])
        if rootpath:
            os.chdir(rootpath)
        self.scheduler.queue.addSubQueue(3, PollEvent.createMatcher(category=PollEvent.WRITE_READY), 'write', None, None, CBQueue.AutoClassQueue.initHelper('fileno'))
        self.scheduler.queue.addSubQueue(1, PollEvent.createMatcher(category=PollEvent.READ_READY), 'read', None, None, CBQueue.AutoClassQueue.initHelper('fileno'))
        self.scheduler.queue.addSubQueue(5, PollEvent.createMatcher(category=PollEvent.ERROR), 'error')
        self.scheduler.queue.addSubQueue(2, ConnectionControlEvent.createMatcher(), 'control')
        self.scheduler.queue.addSubQueue(4, ConnectionWriteEvent.createMatcher(), 'connectionwrite', 40, 40, CBQueue.AutoClassQueue.initHelper('connection'))
        self.scheduler.queue.addSubQueue(10, RoutineControlEvent.createMatcher(), 'routine')
        self.scheduler.queue.addSubQueue(9, TimerEvent.createMatcher(), 'timer')
        self.scheduler.queue.addSubQueue(8, ResolveResponseEvent.createMatcher(), 'resolve')
        self.scheduler.queue.addSubQueue(8, ResolveRequestEvent.createMatcher(), 'resolvereq')
        self.scheduler.queue.addSubQueue(20, SystemControlEvent.createMatcher(), 'sysctl')
        self.scheduler.queue.addSubQueue(0, SystemControlLowPriorityEvent.createMatcher(), 'sysctllow')
        #Client.logger.setLevel('DEBUG')
        #TcpServer.logger.setLevel('DEBUG')
        self.protocolServer = TestProtocol(True)
        self.protocolClient = TestProtocol(False)
        self.resolver = Resolver(scheduler=self.scheduler)
        self.resolver.start()
    def tearDown(self):
        pass

    def testSelfConnection(self):
        c1 = Client('tcp://localhost:199', self.protocolClient, self.scheduler)
        c2 = Client('ptcp://localhost:199', self.protocolServer, self.scheduler)
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        r.main = mainA
        r.start()
        def waitAndStart():
            for m in r.waitWithTimeout(0.5):
                yield m
            c1.start()
        r.subroutine(waitAndStart())
        c2.start()
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')
    
    def testServerClient(self):
        c1 = Client('tcp://localhost:199', self.protocolClient, self.scheduler)
        s1 = TcpServer('ltcp://localhost:199', self.protocolServer, self.scheduler)
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            stopped = False
            while True:
                yield (m,)
                if r.event.connection is c1:
                    ret.extend(b'B')
                else:
                    ret.extend(b'A')
                if not stopped:
                    for m in s1.shutdown():
                        yield m
                    stopped = True
        r.main = mainA
        r.start()
        s1.start()
        def waitAndStart():
            for m in r.waitWithTimeout(0.5):
                yield m
            c1.start()
        r.subroutine(waitAndStart())
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')

    def testMultipleClients(self):
        c1 = Client('tcp://localhost:199', self.protocolClient, self.scheduler)
        c2 = Client('tcp://localhost:199', self.protocolClient, self.scheduler)
        s1 = TcpServer('ltcp://localhost:199', self.protocolServer, self.scheduler)
        r = RoutineContainer(self.scheduler, True)
        counter = {c1:0, c2:0}
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            c1c = False
            c2c = False
            shutdown = False
            while True:
                yield (m,)
                counter[r.event.connection] = counter.get(r.event.connection, 0) + 1
                if r.event.connection is c1:
                    ret.extend(b'A')
                    c1c = True
                elif r.event.connection is c2:
                    ret.extend(b'B')
                    c2c = True
                if c1c and c2c and not shutdown:
                    for m in s1.shutdown():
                        yield m
                    shutdown = True
        r.main = mainA
        r.start()
        s1.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        r.subroutine(waitAndStart(c2))
        self.scheduler.main()
        print(ret)
        self.assertEqual(counter[c1], 10)
        self.assertEqual(counter[c2], 10)
    def testSelfConnectionSsl(self):
        c1 = Client('ssl://localhost:199', self.protocolClient, self.scheduler, 'testcerts/client.key','testcerts/client.crt','testcerts/root.crt')
        c2 = Client('pssl://localhost:199', self.protocolServer, self.scheduler, 'testcerts/server.key','testcerts/server.crt','testcerts/root.crt')
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')
    def testServerClientSsl(self):
        c1 = Client('ssl://localhost:199', self.protocolClient, self.scheduler, 'testcerts/client.key', 'testcerts/client.crt', 'testcerts/root.crt')
        s1 = TcpServer('lssl://localhost:199', self.protocolServer, self.scheduler, 'testcerts/server.key', 'testcerts/server.crt', 'testcerts/root.crt')
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            stopped = False
            while True:
                yield (m,)
                if r.event.connection is c1:
                    ret.extend(b'B')
                else:
                    ret.extend(b'A')
                if not stopped:
                    for m in s1.shutdown():
                        yield m
                    stopped = True
        r.main = mainA
        r.start()
        s1.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')
    def testCAVerify(self):
        c1 = Client('ssl://localhost:199', self.protocolClient, self.scheduler, 'testcerts/selfsigned.key','testcerts/selfsigned.crt','testcerts/selfsigned.crt')
        c2 = Client('pssl://localhost:199', self.protocolServer, self.scheduler, 'testcerts/server.key','testcerts/server.crt','testcerts/root.crt')
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        self.notconnected = False
        def notConnected(connection):
            if connection is c1:
                self.notconnected = True
            if False:
                yield
        self.protocolClient.notconnected = notConnected
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertTrue(self.notconnected)
        self.assertEqual(ret, b'')
    def testCAVerify2(self):
        c1 = Client('ssl://localhost:199', self.protocolClient, self.scheduler, 'testcerts/selfsigned.key','testcerts/selfsigned.crt',None)
        c2 = Client('pssl://localhost:199', self.protocolServer, self.scheduler, 'testcerts/server.key','testcerts/server.crt','testcerts/root.crt')
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        self.notconnected = False
        def notConnected(connection):
            if connection is c1:
                self.notconnected = True
            if False:
                yield
        self.protocolClient.notconnected = notConnected
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertTrue(self.notconnected)
        self.assertEqual(ret, b'')
    def testCAVerify3(self):
        c1 = Client('ssl://localhost:199', self.protocolClient, self.scheduler, None, None, 'testcerts/root.crt')
        c2 = Client('pssl://localhost:199', self.protocolServer, self.scheduler, 'testcerts/server.key','testcerts/server.crt','testcerts/root.crt')
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        self.notconnected = False
        def notConnected(connection):
            if connection is c1:
                self.notconnected = True
            if False:
                yield
        self.protocolClient.notconnected = notConnected
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertTrue(self.notconnected)
        self.assertEqual(ret, b'')
    def testSelfConnectionSslWithoutClientCertificate(self):
        c1 = Client('ssl://localhost:199', self.protocolClient, self.scheduler, None, None,'testcerts/root.crt')
        c2 = Client('pssl://localhost:199', self.protocolServer, self.scheduler, 'testcerts/server.key','testcerts/server.crt',None)
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')
    def testSelfConnectionUdp(self):
        c1 = Client('udp://localhost:199', self.protocolClient, self.scheduler)
        c2 = Client('pudp://localhost:199', self.protocolServer, self.scheduler)
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')
    def testSelfConnectionUnix(self):
        if not hasattr(socket, 'AF_UNIX'):
            print('Skip UNIX socket test because not supported')
            return
        try:
            os.remove('/var/run/unixsocktest.sock')
        except:
            pass
        c1 = Client('unix:/var/run/unixsocktest.sock', self.protocolClient, self.scheduler)
        c2 = Client('punix:/var/run/unixsocktest.sock', self.protocolServer, self.scheduler)
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')
    def testSelfConnectionUnixDgram(self):
        if not hasattr(socket, 'AF_UNIX'):
            print('Skip UNIX socket test because not supported')
            return
        try:
            os.remove('/var/run/unixsocktestudp1.sock')
        except:
            pass
        try:
            os.remove('/var/run/unixsocktestudp2.sock')
        except:
            pass
        c1 = Client('dunix:/var/run/unixsocktestudp2.sock', self.protocolClient, self.scheduler, bindaddress = ((socket.AF_UNIX, '/var/run/unixsocktestudp1.sock'),))
        c2 = Client('pdunix:/var/run/unixsocktestudp2.sock', self.protocolServer, self.scheduler)
        r = RoutineContainer(self.scheduler, True)
        ret = bytearray()
        def mainA():
            m = TestDataEvent.createMatcher()
            while True:
                yield (m,)
                if r.event.connection is c2:
                    ret.extend(b'A')
                else:
                    ret.extend(b'B')
        r.main = mainA
        r.start()
        def waitAndStart(c):
            for m in r.waitWithTimeout(0.5):
                yield m
            c.start()
        r.subroutine(waitAndStart(c1))
        c2.start()
        self.scheduler.main()
        self.assertEqual(ret, b'ABABABABABABABABABAB')

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 2015/6/9

:author: hubo
'''
import unittest
from vlcp.event.event import Event,EventMatcher,withIndices
from vlcp.event.matchtree import MatchTree, EventTree
from vlcp.event.pqueue import CBQueue

@withIndices('type', 'bind', 'number', 'obj')
class TestEvent(Event):
    pass

@withIndices('length')
class TestEvent2(TestEvent):
    pass

@withIndices('type', 'bind', 'number', 'length')
class TestEvent3(Event):
    pass


class Test(unittest.TestCase):


    def setUp(self):
        self.obj = object()
        self.obj2 = object()
        self.tree = MatchTree()
        self.queue = CBQueue(None, None, 5, None)
        self.event1 = TestEvent('READ', ('172.28.98.14',100), 17, self.obj)
        self.event2 = TestEvent('WRITE', ('172.28.0.4',), 17, self.obj2)
        self.event3 = TestEvent('WRITE', ('172.28.0.4',12), 19, self.obj)
        self.event4 = TestEvent('WRITE', ('172.28.0.6',12), 17, self.obj2)
        self.event5 = TestEvent2('WRITE', ('172.28.0.6',12), 17, self.obj2, 27)
        self.event6 = TestEvent3('READ', ('172.28.98.14',100), 17, 27)
        q = self.queue.addSubQueue(1, TestEvent.createMatcher(type='WRITE'), 'write', 15, 20)
        q.addSubQueue(0, TestEvent.createMatcher(bind=('172.28.0.4',)), None, 15, None)
        q = self.queue.addSubQueue(1, TestEvent.createMatcher(type='WRITE', number=19), 'special', 15, 20)
        q = self.queue.addSubQueue(2, TestEvent.createMatcher(type='WRITE', bind=('172.28.98.16',), number=11), 'priority', 15, 15, CBQueue.PriorityQueue)
        q = self.queue.addSubQueue(3, TestEvent.createMatcher(type='WRITE', bind=('172.28.98.17',)), 'autoclass', 15, 15, CBQueue.AutoClassQueue.initHelper('number', 1))
        q = self.queue.addSubQueue(4, TestEvent.createMatcher(type='WRITE', bind=('172.28.98.18',)), 'autoclass2', 15, 15, CBQueue.AutoClassQueue.initHelper('number', 1, subqueuelimit = 5))
        q = self.queue.addSubQueue(4, TestEvent.createMatcher(type='WRITE', bind=('172.28.98.19',)), 'autoclass3', None, None, CBQueue.AutoClassQueue.initHelper('number', 1, subqueuelimit = 5))
        

    def testEventMatcher(self):
        m1 = TestEvent.createMatcher(bind=('172.28.0.4',))
        self.assertFalse(m1.isMatch(self.event1))
        self.assertTrue(m1.isMatch(self.event2))
        self.assertFalse(m1.isMatch(self.event3))
        self.assertFalse(m1.isMatch(self.event4))
        self.assertFalse(m1.isMatch(self.event5))
        self.assertFalse(m1.isMatch(self.event6))
        m2 = TestEvent.createMatcher(type='WRITE', number=17)
        self.assertFalse(m2.isMatch(self.event1))
        self.assertTrue(m2.isMatch(self.event2))
        self.assertFalse(m2.isMatch(self.event3))
        self.assertTrue(m2.isMatch(self.event4))
        self.assertTrue(m2.isMatch(self.event5))
        self.assertFalse(m2.isMatch(self.event6))
        m3 = TestEvent.createMatcher(type='READ', obj=self.obj)
        self.assertTrue(m3.isMatch(self.event1))
        self.assertFalse(m3.isMatch(self.event2))
        self.assertFalse(m3.isMatch(self.event3))
        self.assertFalse(m3.isMatch(self.event4))
        self.assertFalse(m3.isMatch(self.event5))
        self.assertFalse(m3.isMatch(self.event6))
        m4 = TestEvent.createMatcher(type='WRITE',_ismatch=lambda x: x.bind[0] == '172.28.0.4')
        self.assertFalse(m4.isMatch(self.event1))
        self.assertTrue(m4.isMatch(self.event2))
        self.assertTrue(m4.isMatch(self.event3))
        self.assertFalse(m4.isMatch(self.event4))
        self.assertFalse(m4.isMatch(self.event5))
        self.assertFalse(m4.isMatch(self.event6))
        m5 = TestEvent.createMatcher()
        self.assertTrue(m5.isMatch(self.event1))
        self.assertTrue(m5.isMatch(self.event2))
        self.assertTrue(m5.isMatch(self.event3))
        self.assertTrue(m5.isMatch(self.event4))
        self.assertTrue(m5.isMatch(self.event5))
        self.assertFalse(m5.isMatch(self.event6))
        m6 = TestEvent2.createMatcher(type='WRITE', bind=('172.28.0.6',12), obj=self.obj2)
        self.assertFalse(m6.isMatch(self.event1))
        self.assertFalse(m6.isMatch(self.event2))
        self.assertFalse(m6.isMatch(self.event3))
        self.assertFalse(m6.isMatch(self.event4))
        self.assertTrue(m6.isMatch(self.event5))
        self.assertFalse(m6.isMatch(self.event6))
        m7 = TestEvent3.createMatcher(type='READ', number=17)
        self.assertFalse(m7.isMatch(self.event1))
        self.assertFalse(m7.isMatch(self.event2))
        self.assertFalse(m7.isMatch(self.event3))
        self.assertFalse(m7.isMatch(self.event4))
        self.assertFalse(m7.isMatch(self.event5))
        self.assertTrue(m7.isMatch(self.event6))
        m8 = TestEvent2.createMatcher()
        self.assertFalse(m8.isMatch(self.event1))
        self.assertFalse(m8.isMatch(self.event2))
        self.assertFalse(m8.isMatch(self.event3))
        self.assertFalse(m8.isMatch(self.event4))
        self.assertTrue(m8.isMatch(self.event5))
        self.assertFalse(m8.isMatch(self.event6))
        m9 = TestEvent3.createMatcher('READ')
        self.assertFalse(m9.isMatch(self.event1))
        self.assertFalse(m9.isMatch(self.event2))
        self.assertFalse(m9.isMatch(self.event3))
        self.assertFalse(m9.isMatch(self.event4))
        self.assertFalse(m9.isMatch(self.event5))
        self.assertTrue(m9.isMatch(self.event6))
        
    
    def testMatchTree(self):
        self.tree.insert(TestEvent.createMatcher(bind=('172.28.0.4',)), 1)
        self.tree.insert(TestEvent.createMatcher(type='WRITE', number=17), 2)
        self.tree.insert(TestEvent.createMatcher(type='READ', obj=self.obj), 3)
        self.tree.insert(TestEvent.createMatcher(type='WRITE',_ismatch=lambda x: x.bind[0] == '172.28.0.4'), 4)
        self.tree.insert(TestEvent.createMatcher(),5)
        self.tree.insert(TestEvent2.createMatcher(type='WRITE', bind=('172.28.0.6',12), obj=self.obj2), 6)
        self.tree.insert(TestEvent3.createMatcher(type='READ', number=17), 7)
        self.assertEqual(self.tree.matches(self.event1), (3,5))
        self.assertEqual(self.tree.matches(self.event2), (2,4,1,5))
        self.assertEqual(self.tree.matches(self.event3), (4,5))
        self.assertEqual(self.tree.matches(self.event4), (2,5))
        self.assertEqual(self.tree.matches(self.event5), (6,2,5))
        self.assertEqual(self.tree.matches(self.event6), (7,))
        self.tree.remove(TestEvent.createMatcher(type='WRITE',_ismatch=lambda x: x.bind[0] == '172.28.0.4'), 4)
        self.assertEqual(self.tree.matches(self.event1), (3,5))
        self.assertEqual(self.tree.matches(self.event2), (2,1,5))
        self.assertEqual(self.tree.matches(self.event3), (5,))
        self.assertEqual(self.tree.matches(self.event4), (2,5))
        self.assertEqual(self.tree.matches(self.event5), (6,2,5))
        self.assertEqual(self.tree.matches(self.event6), (7,))
        self.tree.insert(TestEvent.createMatcher(type='WRITE',_ismatch=lambda x: x.bind[0] == '172.28.0.4'), 4)
        self.assertEqual(self.tree.matches(self.event1), (3,5))
        self.assertEqual(self.tree.matches(self.event2), (2,4,1,5))
        self.assertEqual(self.tree.matches(self.event3), (4,5))
        self.assertEqual(self.tree.matches(self.event4), (2,5))
        self.assertEqual(self.tree.matches(self.event5), (6,2,5))
        self.assertEqual(self.tree.matches(self.event6), (7,))
    
    @staticmethod
    def popAll(queue):
        while queue.canPop():
            yield queue.pop()
    @staticmethod
    def pushAll(queue, events):
        while events:
            w = queue.append(events[0])
            if w is not None:
                return w
            del events[0]
        return None

    def testQueuePriority(self):
        events = []
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('READ', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.4',),19,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),19,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):
            e = TestEvent('WRITE', ('172.28.98.16',),11,self.obj, priority = 10 - (i * 2) % 5)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [27,29,26,28,25,5,20,10,21,6,22,11,23,7,24,12,8,13,9,14,15,16,17,18,19,0,1,2,3,4])
    
    def testQueueFull(self):
        events = []
        for i in range(0,20):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 5)
        self.assertTrue(ans is not None and isinstance(ans, EventMatcher) and ans.indices[1] is self.queue['write'].defaultQueue)
        (result, we, es) = self.queue.pop()
        self.assertEqual(result.testIndex, 0)
        self.assertEqual(len(we), 1)
        self.assertTrue(we[0].queue is self.queue['write'].defaultQueue)
        self.assertTrue(ans.isMatch(we[0]))
        ans = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 4)
        self.assertTrue(ans is not None and isinstance(ans, EventMatcher))
        events = []
        for i in range(0,20):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.4',), 17, self.obj)
            e.testIndex = len(events) + 20
            events.append(e)
        ans2 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        self.assertTrue(ans is not None and isinstance(ans, EventMatcher))
        (result, we, es) = self.queue.pop()
        self.assertTrue(len(we) >= 1)
        self.assertTrue(ans2.isMatch(we[0]) or ans2.isMatch(we[1]))
        events = []
        for i in range(0,20):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.98.16',), 11, self.obj, priority = (i * 4) % 21)
            e.testIndex = len(events) + 40
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 5)
        self.assertTrue(ans is not None and isinstance(ans, EventMatcher))
        (result, we, es) = self.queue.pop()
        self.assertEqual(len(we), 1)
        self.assertTrue(ans.isMatch(we[0]))
        
    
    def testQueueBlock(self):
        events = []
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('READ', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        (ret, ws, es) = self.queue.pop()
        self.assertTrue(not ws)
        self.assertEqual(ret.testIndex, 5)
        self.queue.block(ret)
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [10,11,12,13,14,0,1,2,3,4])
        self.queue.unblock(ret)
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [5,6,7,8,9])
        events = []
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('READ', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        (ret, ws, es) = self.queue.pop()
        self.assertTrue(not ws)
        self.assertEqual(ret.testIndex, 5)
        self.queue.block(ret)
        (ret2, ws, es) = self.queue.pop()
        self.assertTrue(not ws)
        self.assertEqual(ret2.testIndex, 0)
        self.queue.block(ret2)
        self.assertFalse(self.queue.canPop())
        self.assertEqual(len(self.queue), 10)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events) + 10
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)        
        self.assertFalse(self.queue.canPop())
        self.queue.unblockall()
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [5,6,7,8,9,10,11,12,13,14,0,1,2,3,4])
        events = []
        for i in range(0,5):
            e = TestEvent('WRITE', ('172.28.98.16',),11,self.obj, priority = 10 - (i * 2) % 5)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        (ret, ws, es) = self.queue.pop()
        self.assertTrue(not ws)
        self.assertEqual(ret.testIndex, 2)
        self.queue.block(ret)
        self.assertFalse(self.queue.canPop())
        for i in range(0,5):
            e = TestEvent('WRITE', ('172.28.98.16',),11,self.obj, priority = 5 - (i * 2) % 5)
            e.testIndex = len(events) + 5
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        (ret2, ws, es) = self.queue.pop()
        self.assertTrue(not ws)
        self.assertEqual(ret2.testIndex, 7)
        self.queue.block(ret2)
        self.assertFalse(self.queue.canPop())
        for i in range(0,5):
            e = TestEvent('WRITE', ('172.28.98.16',),11,self.obj, priority = - ((i * 2) % 5))
            e.testIndex = len(events) + 10
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [12,14,11,13,10])
        self.queue.unblock(ret)
        self.assertFalse(self.queue.canPop())
        self.queue.unblock(ret2)
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [7,9,6,8,5,2,4,1,3,0])
        
        
    def testClear(self):
        events = []
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('READ', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.4',),19,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),19,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):
            e = TestEvent('WRITE', ('172.28.98.16',),11,self.obj, priority = 10 - (i * 2) % 5)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        (ret, ws, es) = self.queue.pop()
        self.assertTrue(not ws)
        self.assertEqual(ret.testIndex, 27)
        self.queue.block(ret)
        self.queue.clear()
        self.assertEqual(len(self.queue), 0)
        self.assertFalse(self.queue.canPop())
    
    def testCircleList(self):
        cl = CBQueue.MultiQueue.CircleList()
        n = CBQueue.MultiQueue.CircleListNode(1)
        n2 = CBQueue.MultiQueue.CircleListNode(2)
        n3 = CBQueue.MultiQueue.CircleListNode(3)
        cl.insertprev(n)
        self.assertTrue(cl.current is n)
        cl.insertprev(n2)
        self.assertTrue(cl.current is n)
        cl.remove(n)
        self.assertTrue(cl.current is n2)
        cl.insertprev(n3)
        self.assertTrue(cl.next() is n2)
        self.assertTrue(cl.next() is n3)
        self.assertTrue(cl.next() is n2)
        self.assertTrue(cl.current is n3)
        cl.remove(n2)
        self.assertTrue(cl.current is n3)
        cl.clear()
        self.assertTrue(cl.current is None)
        
    def testRemoveQueue(self):
        events = []
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('READ', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        self.queue.removeSubQueue('write')
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [0,1,2,3,4,5,6,7,8,9])
    
    def testSetPriority(self):
        events = []
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('READ', ('172.28.0.4',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),17,self.obj)
            e.testIndex = len(events)
            events.append(e)
        for i in range(0,5):  # @UnusedVariable
            e = TestEvent('WRITE', ('172.28.0.6',),19,self.obj)
            e.testIndex = len(events)
            events.append(e)
        ans = self.pushAll(self.queue, events)
        self.assertTrue(not events and ans is None)
        self.queue.setPriority('special', 3)
        eventOut = [e[0].testIndex for e in self.popAll(self.queue)]
        self.assertEqual(eventOut, [10,11,12,13,14,5,6,7,8,9,0,1,2,3,4])
    
    def testEventTree(self):
        def createTree():
            tree = EventTree()
            tree.insert(self.event1)
            tree.insert(self.event2)
            tree.insert(self.event3)
            tree.insert(self.event4)
            tree.insert(self.event5)
            tree.insert(self.event6)
            return tree
        t = createTree()
        self.assertTrue(hasattr(t,'index'))
        self.assertEqual(t.findAndRemove(TestEvent.createMatcher(bind=('172.28.0.4',))), (self.event2,))
        self.assertEqual(t.findAndRemove(TestEvent.createMatcher(bind=('172.28.0.4',))), ())
        t = createTree()
        self.assertEqual(t.findAndRemove(TestEvent.createMatcher(type='WRITE', number=17)), (self.event2, self.event4, self.event5))
        self.assertEqual(t.findAndRemove(TestEvent.createMatcher(type='WRITE', number=17)), ())
        self.assertEqual(t.findAndRemove(TestEvent.createMatcher(type='READ', obj=self.obj)), (self.event1,))
    
    def testEmptyEvent(self):
        self.queue.append(self.event1)
        m = self.queue.waitForEmpty()
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(len(es), 1)
        self.assertTrue(m.isMatch(es[0]))
        self.queue.block(ret, es)
        self.queue.unblock(ret)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(len(es), 1)
        self.assertTrue(m.isMatch(es[0]))
    
    def testAutoClassQueue(self):
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.17',), 10, self.obj, testid = i))
        ans = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 6)
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.17',), 20, self.obj, testid = i + 20))
        ans2 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 19)
        (ret, ws, es) = self.queue.pop()
        self.assertFalse(ws)
        self.assertEqual(ret.testid, 0)
        ans3 = self.queue.append(TestEvent('WRITE', ('172.28.98.17',), 30, self.obj2, testid = 40))
        self.assertTrue(ans3 is None)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 40)
        self.assertFalse(ws)
        ans4 = self.queue.append(TestEvent('WRITE', ('172.28.98.17',), 30, self.obj2, testid = 41))
        self.assertTrue(ans4 is not None)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 20)
        self.assertEqual(len(ws), 1)
        self.assertTrue(ans2.isMatch(ws[0]))
        self.assertFalse(ans.isMatch(ws[0]))
        self.assertTrue(ans4.isMatch(ws[0]))
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 1)
        self.assertFalse(ws)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 2)
        self.assertEqual(len(ws), 1)
        self.assertTrue(ans.isMatch(ws[0]))
        blk = ret
        self.queue.block(ret, es)
        ans2 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 18)
        ans3 = self.queue.append(TestEvent('WRITE', ('172.28.98.17',), 30, self.obj2, testid = 42))
        self.assertTrue(ans3 is None)
        ans4 = self.queue.append(TestEvent('WRITE', ('172.28.98.17',), 30, self.obj2, testid = 43))
        self.assertTrue(ans4 is not None)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 42)
        self.assertEqual(len(ws), 1)
        self.assertTrue(ans4.isMatch(ws[0]) and not ans2.isMatch(ws[0]) and not ans.isMatch(ws[0]))
        self.queue.unblock(blk)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 21)
        self.assertEqual(len(ws), 1)
        self.assertTrue(not ans4.isMatch(ws[0]) and ans2.isMatch(ws[0]) and not ans.isMatch(ws[0]))
    def testAutoClassQueueWithSubQueueLimit(self):
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.18',), 10, self.obj, testid = i))
        ans = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.18',), 20, self.obj, testid = i + 20))
        ans2 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(len(ws), 1)
        self.assertTrue(ans.isMatch(ws[0]))
        self.assertFalse(ans2.isMatch(ws[0]))
        self.assertEqual(ret.testid, 0)
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.18',), 30, self.obj, testid = i + 40))
        ans3 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 40)
        self.assertEqual(len(ws), 1)
        self.assertFalse(ans.isMatch(ws[0]))
        self.assertFalse(ans2.isMatch(ws[0]))
        self.assertTrue(ans3.isMatch(ws[0]))
        ans3 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 14)
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.18',), 40, self.obj, testid = i + 60))
        ans4 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 19)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 60)
        self.assertEqual(len(ws), 1)
        self.assertFalse(ans.isMatch(ws[0]))
        self.assertFalse(ans2.isMatch(ws[0]))
        self.assertFalse(ans3.isMatch(ws[0]))
        self.assertTrue(ans4.isMatch(ws[0]))        
        ans4 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 19)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 20)
        self.assertEqual(len(ws), 1)
        self.assertFalse(ans.isMatch(ws[0]))
        self.assertTrue(ans2.isMatch(ws[0]))
        self.assertFalse(ans3.isMatch(ws[0]))
        self.assertTrue(ans4.isMatch(ws[0]))
        (ws, es) = self.queue.clear()
        self.assertEqual(len(ws), 1)
        self.assertTrue(ans.isMatch(ws[0]))
        self.assertTrue(ans2.isMatch(ws[0]))
        self.assertTrue(ans3.isMatch(ws[0]))
        self.assertTrue(ans4.isMatch(ws[0]))
    def testAutoClassQueueWithoutMainLimit(self):
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.19',), 10, self.obj, testid = i))
        ans = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.19',), 20, self.obj, testid = i + 20))
        ans2 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(len(ws), 1)
        self.assertTrue(ans.isMatch(ws[0]))
        self.assertFalse(ans2.isMatch(ws[0]))
        self.assertEqual(ret.testid, 0)
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.19',), 30, self.obj, testid = i + 40))
        ans3 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 40)
        self.assertEqual(len(ws), 1)
        self.assertFalse(ans.isMatch(ws[0]))
        self.assertFalse(ans2.isMatch(ws[0]))
        self.assertTrue(ans3.isMatch(ws[0]))
        ans3 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 14)
        events = []
        for i in range(0, 20):
            events.append(TestEvent('WRITE', ('172.28.98.19',), 40, self.obj, testid = i + 60))
        ans4 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 15)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 60)
        self.assertEqual(len(ws), 1)
        self.assertFalse(ans.isMatch(ws[0]))
        self.assertFalse(ans2.isMatch(ws[0]))
        self.assertFalse(ans3.isMatch(ws[0]))
        self.assertTrue(ans4.isMatch(ws[0]))        
        ans4 = self.pushAll(self.queue, events)
        self.assertEqual(len(events), 14)
        (ret, ws, es) = self.queue.pop()
        self.assertEqual(ret.testid, 20)
        self.assertEqual(len(ws), 1)
        self.assertFalse(ans.isMatch(ws[0]))
        self.assertTrue(ans2.isMatch(ws[0]))
        self.assertFalse(ans3.isMatch(ws[0]))
        self.assertFalse(ans4.isMatch(ws[0]))
        (ws, es) = self.queue.clear()
        self.assertEqual(len(ws), 1)
        self.assertTrue(ans.isMatch(ws[0]))
        self.assertTrue(ans2.isMatch(ws[0]))
        self.assertTrue(ans3.isMatch(ws[0]))
        self.assertTrue(ans4.isMatch(ws[0]))

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testEvent']
    unittest.main()




'''
Created on 2015/8/27

:author: hubo
'''
from __future__ import print_function
from vlcp.server import Server
from vlcp.event import RoutineContainer, Stream, TcpServer, MemoryStream, Client
from vlcp.protocol.http import Http, HttpRequestEvent, HttpConnectionStateEvent
from codecs import getincrementalencoder
import logging

http = Http(False)

class MainRoutine(RoutineContainer):
    def __init__(self, scheduler=None, daemon=False):
        RoutineContainer.__init__(self, scheduler=scheduler, daemon=daemon)
    def main(self):
        conn = Client('tcp://www.baidu.com/', http, self.scheduler)
        conn.start()
        connected = http.statematcher(conn, HttpConnectionStateEvent.CLIENT_CONNECTED, False)
        notconnected = http.statematcher(conn, HttpConnectionStateEvent.CLIENT_NOTCONNECTED, False)
        yield (connected, notconnected)
        if self.matcher is notconnected:
            print('Connect to server failed.')
        else:
            for m in http.requestwithresponse(self, conn, b'www.baidu.com', b'/', b'GET', []):
                yield m
            for r in self.http_responses:
                print('Response received:')
                print(r.status)
                print()
                print('Headers:')
                for k,v in r.headers:
                    print('%r: %r' % (k,v))
                print()
                print('Body:')
                if r.stream is None:
                    print('<Empty>')
                else:
                    try: 
                        while True:
                            for m in r.stream.read(self, 1024):
                                yield m
                            print(self.data, end = '')
                    except EOFError:
                        pass
                    print()
            for m in http.requestwithresponse(self, conn, b'www.baidu.com', b'/favicon.ico', b'GET', [], keepalive = False):
                yield m
            for r in self.http_responses:
                print('Response received:')
                print(r.status)
                print()
                print('Headers:')
                for k,v in r.headers:
                    print('%r: %r' % (k,v))
                print()
                print('Body:')
                if r.stream is None:
                    print('<Empty>')
                else:
                    for m in r.stream.read(self):
                        yield m
                    print('<Data: %d bytes>' % (len(self.data),))

if __name__ == '__main__':
    logging.basicConfig()
    s = Server()
    s.scheduler.debugging = True
    s.scheduler.logger.setLevel(logging.DEBUG)
    #Http.debugging = True
    #Http._logger.setLevel(logging.DEBUG)
    http.createmessagequeue(s.scheduler)
    routine = MainRoutine(s.scheduler)
    routine.start()
    s.serve()
    



'''
Created on 2015/6/18

:author: hubo
'''
import unittest
from vlcp.event.core import Scheduler
from vlcp.event.runnable import RoutineContainer, RoutineControlEvent
from vlcp.event.event import Event, withIndices
from vlcp.event.connection import Client, TcpServer
import logging

@withIndices('producer')
class TestConsumerEvent(Event):
    canignore = False



class Test(unittest.TestCase):

    def setUp(self):
        logging.basicConfig()
    def testConsumer(self):
        scheduler = Scheduler()
        scheduler.queue.addSubQueue(10, RoutineControlEvent.createMatcher())
        scheduler.queue.addSubQueue(1, TestConsumerEvent.createMatcher(), 'consumer', 5, 5)
        rA = RoutineContainer(scheduler)
        output = bytearray()
        def mainA():
            rA.subroutine(mainB(), daemon = True)
            for i in range(0,10):
                for ms in rA.waitForSend(TestConsumerEvent(rA.mainroutine)):
                    yield ms
                output.extend(b'A')
        def mainB():
            matcher = TestConsumerEvent.createMatcher(producer=rA.mainroutine)
            while True:
                yield (matcher,)
                rA.event.canignore = True
                output.extend(b'B')
        rA.main = mainA
        rA.start()
        scheduler.main()
        self.assertEqual(output, b'AAAAABABABABABABBBBB')
    def testLoopConsumer(self):
        scheduler = Scheduler()
        scheduler.queue.addSubQueue(10, RoutineControlEvent.createMatcher())
        scheduler.queue.addSubQueue(1, TestConsumerEvent.createMatcher(), 'consumer', 5, 5)
        rA = RoutineContainer(scheduler)
        output = bytearray()
        def mainA():
            rA.subroutine(mainB(), True, 'mainB', True)
            matcher = TestConsumerEvent.createMatcher(rA.mainB)
            for i in range(0,10):
                for ms in rA.waitForSend(TestConsumerEvent(rA.mainroutine)):
                    yield ms
                output.extend(b'A')
                yield (matcher,)
        def mainB():
            matcher = TestConsumerEvent.createMatcher(producer=rA.mainroutine)
            while True:
                yield (matcher,)
                rA.event.canignore = True
                output.extend(b'B')
                for ms in rA.waitForSend(TestConsumerEvent(rA.mainB, canignore = True)):
                    yield ms                
        rA.main = mainA
        rA.start()
        scheduler.main()
        self.assertEqual(output, b'ABABABABABABABABABAB')
    def testBlock(self):
        scheduler = Scheduler()
        scheduler.queue.addSubQueue(10, RoutineControlEvent.createMatcher())
        scheduler.queue.addSubQueue(1, TestConsumerEvent.createMatcher(), 'consumer', 5, 5)
        rA = RoutineContainer(scheduler)
        output = bytearray()
        def mainA():
            rA.subroutine(mainB(), daemon = True)
            for i in range(0,10):
                for ms in rA.waitForSend(TestConsumerEvent(rA.mainroutine)):
                    yield ms
                output.extend(b'A')
        def mainB():
            for m in rA.doEvents():
                yield m
            matcher = TestConsumerEvent.createMatcher(producer=rA.mainroutine)
            while True:
                yield (matcher,)
                rA.event.canignore = True
                output.extend(b'B')
        def mainC():
            for m in rA.doEvents():
                yield m
            output.extend(b'C')
        rA.main = mainA
        rA.start()
        rA.subroutine(mainC())
        scheduler.main()
        self.assertEqual(output, b'AAAAACBABABABABABBBBB')

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testConsumer']
    unittest.main()



'''
Created on 2015/8/27

:author: hubo
'''
from pprint import pprint
from vlcp.server import main
from vlcp.server.module import Module, depend
import vlcp.service.utils.session
import vlcp.service.connection.httpserver
import vlcp.service.web.static
from vlcp.utils.http import HttpHandler
from vlcp.config.config import manager
import logging

@depend(vlcp.service.connection.httpserver.HttpServer, vlcp.service.utils.session.Session,
        vlcp.service.web.static.Static)
class TestHttpServer(Module):
    def __init__(self, server):
        Module.__init__(self, server)
        self.routines.append(MainHandler(self.scheduler))

class MainHandler(HttpHandler):
    document = '''
<!DOCTYPE html >
<html>
<head>
<title>Test Server Page</title>
<link rel="stylesheet" type="text/css" href="/static/test.css"/>
</head>
<body>
OK!<br/>
Host = %s<br/>
Path = %s<br/>
Headers = %s<br/>
RealPath = %s<br/>
OriginalPath = %s<br/>
Cookies = %s<br/>
Args = %s<br/>
Form = %s<br/>
</body>
</html>
'''
    rewrites = ((br'/test1', br'/?test1=true'),
                (br'/test/(.*)', br'/?test=\1'),
                (br'/test3/(.*)', br'../?test=\1'),
                (br'/testcss', br'/static/test.css'))
    redirects = ((br'/test2', br'/?test2=true'),
                 (br'/redirect/(.*)', br'/\1'),
                 (br'/redirect2/(.*)', br'../test/\1'))
    def formatstr(self, tmpl, params):
        if not isinstance(tmpl, bytes):
            # Python 3
            return (tmpl % tuple(v if not hasattr(v, 'decode') else v.decode('utf-8') for v in params))
        else:
            return (tmpl % params)
    @HttpHandler.route(b'/', method = [b'GET', b'HEAD', b'POST'])
    def default(self, env):
        for m in env.parseform():
            yield m
        #for m in self.waitWithTimeout(10):
        #    yield m
        if 'auth' in env.args:
            username, password = env.basicauth()
            if username != b'test' or password != b'testpassword':
                env.basicauthfail()
        elif 'exception' in env.args:
            raise Exception('Test Exception')
        elif 'rewrite' in env.args:
            for m in env.rewrite(b'/?test=a&test2=b'):
                yield m
            env.exit()
        elif 'redirect' in env.args:
            for m in env.redirect(b'/?test3=b&test4=c'):
                yield m
            env.exit()
        elif 'redirect2' in env.args:
            for m in env.redirect(b'http://www.baidu.com/'):
                yield m
            env.exit()
        if 'download' in env.args:
            env.header('Content-Disposition', 'attachment; filename="a:b.txt"')
        for m in env.write(self.formatstr(self.document,
                                          (env.escape(env.host),
                                           env.escape(env.fullpath),
                                           env.escape(repr(env.headers)),
                                           env.escape(env.path),
                                           env.escape(env.originalpath),
                                           env.escape(repr(env.cookies)),
                                           env.escape(repr(env.args)),
                                           env.escape(repr(env.form))
                                           ))):
            yield m

if __name__ == '__main__':
    #s.scheduler.debugging = True
    #s.scheduler.logger.setLevel(logging.DEBUG)
    #Http.debugging = True
    #Http._logger.setLevel(logging.DEBUG)
    #manager['server.debugging'] = True
    manager['module.httpserver.url'] = None
    manager['module.httpserver.urls'] = ['ltcp://0.0.0.0:8080']
    manager['protocol.http.showerrorinfo'] = True
    manager['module.console.startinconsole'] = False
    main(None, ('__main__.TestHttpServer',))
    



'''
Created on 2015/9/28

:author: hubo
'''

from __future__ import print_function, division
from vlcp.protocol import Protocol
from time import time
from vlcp.event import ConnectionWriteEvent, Event, withIndices, RoutineContainer, Client, TcpServer
import argparse
import sys
from vlcp.server import Server
import logging
from vlcp.event.core import TimerEvent
from vlcp.config.config import defaultconfig

@withIndices('state', 'connection')
class TestConnectionEvent(Event):
    UP = 'up'
    DOWN = 'down'

@defaultconfig
class TcpTestProtocol(Protocol):
    _default_buffersize = 1048576
    _default_totalsend = 10
    def __init__(self, server = False):
        self.server = server
    def init(self, connection):
        for m in Protocol.init(self, connection):
            yield m
        if not self.server:
            connection.subroutine(connection.executeWithTimeout(self.totalsend + 1.0, self._clientroutine(connection)), False, 'protocolroutine')
        else:
            for m in connection.write(ConnectionWriteEvent(connection, connection.connmark, data = b'', EOF = True)):
                yield m
        for m in connection.waitForSend(TestConnectionEvent(TestConnectionEvent.UP, connection)):
            yield m
    def closed(self, connection):
        for m in Protocol.closed(self, connection):
            yield m
        for m in connection.waitForSend(TestConnectionEvent(TestConnectionEvent.DOWN, connection)):
            yield m
    def error(self, connection):
        for m in Protocol.closed(self, connection):
            yield m
        for m in connection.waitForSend(TestConnectionEvent(TestConnectionEvent.DOWN, connection)):
            yield m
    def _clientroutine(self, connection):
        # Send Data Until Connection closed
        try:
            data = b'\x00' * self.buffersize
            while True:
                we = ConnectionWriteEvent(connection, connection.connmark, data = data)
                for m in connection.write(we, False):
                    yield m
        except:
            for m in connection.shutdown(True):
                yield m
    def parse(self, connection, data, laststart):
        return ([], 0)
    
class Sampler(RoutineContainer):
    def __init__(self, scheduler = None, interval = 10.0, server = False):
        RoutineContainer.__init__(self, scheduler, True)
        self.connections = {}
        self.identifiers = {}
        self.idg = 0
        self.interval = interval
        self.server = server
    def main(self):
        em = TestConnectionEvent.createMatcher()
        self.sampler = None
        while True:
            yield (em,)
            if self.event.state == TestConnectionEvent.UP:
                if self.server:
                    self.connections[self.event.connection] = self.event.connection.totalrecv
                else:
                    self.connections[self.event.connection] = self.event.connection.totalsend
                identifier = '[%3d]' % (self.event.connection.socket.fileno(),)
                self.identifiers[self.event.connection] = identifier
                print('%s Connected: %r' % (identifier, self.event.connection))
                if not self.sampler:
                    self.subroutine(self._sampleroutine(), True, 'sampler', True)
            else:
                print('%s Disconnected' % (self.identifiers[self.event.connection]))
                del self.connections[self.event.connection]
                del self.identifiers[self.event.connection]
                if not self.connections:
                    self.terminate(self.sampler)
                    self.sampler = None
    def _sampleroutine(self):
        th = self.scheduler.setTimer(self.interval, self.interval)
        try:
            tm = TimerEvent.createMatcher(th)
            lt = time()
            t = 0
            interval = self.interval
            while True:
                yield (tm,)
                ct = time()
                tc = 0
                if self.connections:
                    for connection,lc in list(self.connections.items()):
                        if self.server:
                            cc = connection.totalrecv
                        else:
                            cc = connection.totalsend
                        speed = (cc - lc) / (ct - lt)
                        tc += (cc - lc)
                        unit = ''
                        if speed > 1024.0:
                            speed /= 1024.0
                            unit = 'K'
                            if speed > 1024.0:
                                speed /= 1024.0
                                unit = 'M'
                                if speed > 1024.0:
                                    speed /= 1024.0
                                    unit = 'G'
                        print('%s\t%.1fs - %.1fs\t%.2f%sB/s(%.2f%sbit/s)' % (self.identifiers[connection], t * interval, (t+1) * interval, speed, unit, speed * 8, unit))
                        self.connections[connection] = cc
                    if len(self.connections) > 1:
                        speed = tc / (ct - lt)
                        unit = ''
                        if speed > 1024.0:
                            speed /= 1024.0
                            unit = 'K'
                            if speed > 1024.0:
                                speed /= 1024.0
                                unit = 'M'
                                if speed > 1024.0:
                                    speed /= 1024.0
                                    unit = 'G'
                        print('[SUM]\t%.1fs - %.1fs\t%.2f%sB/s(%.2f%sbit/s)' % (t * interval, (t+1) * interval, speed, unit, speed * 8, unit))
                lt = ct
                t += 1
        finally:
            self.scheduler.cancelTimer(th)

if __name__ == '__main__':
    logging.basicConfig()
    parse = argparse.ArgumentParser(description='Test TCP IO Bandwidth')
    parse.add_argument('-s', '--server', action='store_true', help='Start in server mode')
    parse.add_argument('-c', '--client', help='Start in client mode and connect to IP', metavar='IP')
    parse.add_argument('-i', '--interval', type=float, help='Display bandwidth in interval', default = 10.0)
    parse.add_argument('-t', '--time', type=float, help='Run specified time', default = 10.0)
    parse.add_argument('-B', '--bind', help='When in server mode, bind to specified address', default='0.0.0.0')
    parse.add_argument('-p', '--port', type=int, help='When in server mode, bind to specified port (default 5987)', default=5987)
    parse.add_argument('-P', '--parallel', type=int, help='When in client mode, start multiple clients', metavar = 'NUM', default=1)
    args = parse.parse_args()
    s = Server()
    if args.server and args.client:
        print('Cannot specify both --server and --client')
        sys.exit(1)
    elif not args.server and not args.client:
        print('Must specify either --server or --client')
        sys.exit(1)
    elif args.server:
        tp = TcpTestProtocol(True)
        conn = TcpServer('tcp://%s:%d/' % (args.bind, args.port), tp, s.scheduler)
        sampler = Sampler(s.scheduler, args.interval, True)
        conn.start()
        sampler.start()
    else:
        tp = TcpTestProtocol(False)
        tp.totalsend = args.time
        sampler = Sampler(s.scheduler, args.interval, False)
        for _ in range(0, args.parallel):
            conn = Client('tcp://%s:%d/' % (args.client, args.port), tp, s.scheduler)
            conn.start()
        sampler.start()
    s.serve()



'''
Created on 2015/10/14

:author: hubo
'''
import unittest
from vlcp.server import Server
from vlcp.server.module import callAPI, ModuleLoadStateChanged
import logging
import os.path
from vlcp.event.runnable import RoutineContainer
from vlcp.config import manager
from vlcp.utils.pycache import removeCache
import traceback

try:
    reload
except:
    from vlcp.utils.pycache import reload

module1 = b'''
from vlcp.server.module import Module, api
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer

@withIndices()
class ModuleTestEvent(Event):
    pass

@defaultconfig
class TestModule1(Module):
    class MyHandler(RoutineContainer):
        def method2(self, a, b):
            "Run method2"
            self.retvalue = a + b
            if False:
                yield
        def method3(self, a, b):
            "Run method3"
            for m in self.waitForSend(ModuleTestEvent(a = a, b = b)):
                yield m
            self.retvalue = None
    def __init__(self, server):
        Module.__init__(self, server)
        self.handlerRoutine = self.MyHandler(self.scheduler)
        self.createAPI(api(self.method1),
                api(self.handlerRoutine.method2,self.handlerRoutine),
                api(self.handlerRoutine.method3,self.handlerRoutine),
                api(self.method4)
                )
    def method1(self):
        "Run method1"
        return 'version1'
    def method4(self):
        "Run method4"
        raise ValueError('test')
'''

module2 = b'''
from vlcp.server.module import Module, api, depend
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer
from . import testmodule1

@withIndices()
class ModuleTestEvent2(Event):
    pass

@defaultconfig
@depend(testmodule1.TestModule1)
class TestModule2(Module):
    class MyHandler(RoutineContainer):
        def main(self):
            matcher = testmodule1.ModuleTestEvent.createMatcher()
            while True:
                yield (matcher,)
                self.subroutine(self.waitForSend(ModuleTestEvent2(result=self.event.a + self.event.b, version = 'version1')), False)
    def __init__(self, server):
        Module.__init__(self, server)
        self.routines.append(self.MyHandler(self.scheduler))
'''

module1v2 = b'''
from vlcp.server.module import Module, api
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer

@withIndices()
class ModuleTestEvent(Event):
    pass

@defaultconfig
class TestModule1(Module):
    class MyHandler(RoutineContainer):
        def method2(self, a, b):
            self.retvalue = a + b
            if False:
                yield
        def method3(self, a, b):
            for m in self.waitForSend(ModuleTestEvent(a = a, b = b)):
                yield m
            self.retvalue = None
    def __init__(self, server):
        Module.__init__(self, server)
        self.handlerRoutine = self.MyHandler(self.scheduler)
        self.createAPI(api(self.method1),
                api(self.handlerRoutine.method2,self.handlerRoutine),
                api(self.handlerRoutine.method3,self.handlerRoutine),
                api(self.method4)
                )
    def method1(self):
        return 'version2'
    def method4(self):
        raise ValueError('test')
'''

module2v2 = b'''
from vlcp.server.module import Module, api, depend
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer
from . import testmodule1

@withIndices()
class ModuleTestEvent2(Event):
    pass

@defaultconfig
@depend(testmodule1.TestModule1)
class TestModule2(Module):
    class MyHandler(RoutineContainer):
        def main(self):
            matcher = testmodule1.ModuleTestEvent.createMatcher()
            while True:
                yield (matcher,)
                self.subroutine(self.waitForSend(ModuleTestEvent2(result=self.event.a + self.event.b, version = 'version2')), False)
    def __init__(self, server):
        Module.__init__(self, server)
        self.routines.append(self.MyHandler(self.scheduler))
'''


module1v3 = b'''
from vlcp.server.module import Module, api
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer

@withIndices()
class ModuleTestEvent(Event):
    pass

@defaultconfig
class TestModule1(Module):
    class MyHandler(RoutineContainer):
        def method2(self, a, b):
            self.retvalue = a + b
            if False:
                yield
        def method3(self, a, b):
            for m in self.waitForSend(ModuleTestEvent(a = a, b = b)):
                yield m
            self.retvalue = None
    def __init__(self, server):
        Module.__init__(self, server)
        self.handlerRoutine = self.MyHandler(self.scheduler)
        self.createAPI(api(self.method1),
                api(self.handlerRoutine.method2,self.handlerRoutine),
                api(self.handlerRoutine.method3,self.handlerRoutine),
                api(self.method4)
                )
    def method1(self):
        return 'version3'
    def method4(self):
        raise ValueError('test')
'''

module2v3 = b'''
from vlcp.server.module import Module, api, depend
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices
from vlcp.event.runnable import RoutineContainer
from . import testmodule1

@withIndices()
class ModuleTestEvent2(Event):
    pass

@defaultconfig
@depend(testmodule1.TestModule1)
class TestModule2(Module):
    class MyHandler(RoutineContainer):
        def main(self):
            matcher = testmodule1.ModuleTestEvent.createMatcher()
            while True:
                yield (matcher,)
                self.subroutine(self.waitForSend(ModuleTestEvent2(result=self.event.a + self.event.b, version = 'version3')), False)
    def __init__(self, server):
        Module.__init__(self, server)
        self.routines.append(self.MyHandler(self.scheduler))
'''



class Test(unittest.TestCase):
    def testModuleLoad(self):
        logging.basicConfig()
        manager['server.startup'] = ('tests.gensrc.testmodule1.TestModule1', 'tests.gensrc.testmodule2.TestModule2')
        s = Server()
        import tests.gensrc
        basedir = tests.gensrc.__path__[0]
        with open(os.path.join(basedir, 'testmodule1.py'), 'wb') as f:
            f.write(module1)
        with open(os.path.join(basedir, 'testmodule2.py'), 'wb') as f:
            f.write(module2)
        # Run unittest discover may already load the module, reload it
        import tests.gensrc.testmodule1
        import tests.gensrc.testmodule2
        removeCache(tests.gensrc.testmodule1)
        removeCache(tests.gensrc.testmodule2)
        reload(tests.gensrc.testmodule1)
        reload(tests.gensrc.testmodule2)
        # Sometimes the timestamp is not working, make sure python re-compile the source file
        r = RoutineContainer(s.scheduler)
        apiResults = []
        def testproc():
            yield (ModuleLoadStateChanged.createMatcher(),)
            for m in callAPI(r, "testmodule1", "method1", {}):
                yield m
            apiResults.append(r.retvalue)
            for m in callAPI(r, "testmodule1", "method2", {'a' : 1, 'b' : 2}):
                yield m
            apiResults.append(r.retvalue)
            try:
                for m in callAPI(r, "testmodule1", "method4", {}):
                    yield m
                apiResults.append(None)
            except ValueError as exc:
                apiResults.append(exc.args[0])
            from .gensrc.testmodule2 import ModuleTestEvent2
            matcher = ModuleTestEvent2.createMatcher()
            self.event = False
            def proc2():            
                for m in callAPI(r, "testmodule1", "method3", {'a' : 1, 'b' : 2}):
                    yield m
            def callback(event, matcher):
                self.event = event
                if False:
                    yield
            for m in r.withCallback(proc2(), callback, matcher):
                yield m
            if not self.event:
                for m in r.waitWithTimeout(0.1, matcher):
                    yield m
                if not r.timeout:
                    self.event = r.event
            if self.event:
                apiResults.append((self.event.result, self.event.version))
            else:
                apiResults.append(False)
            for m in callAPI(r, "testmodule1", "discover", {}):
                yield m
            apiResults.append(r.retvalue)
            with open(os.path.join(basedir, 'testmodule1.py'), 'wb') as f:
                f.write(module1v2)
            for m in s.moduleloader.delegate(s.moduleloader.reloadModules(['tests.gensrc.testmodule1.TestModule1'])):
                yield m
            for m in callAPI(r, "testmodule1", "method1", {}):
                yield m
            apiResults.append(r.retvalue)
            matcher = ModuleTestEvent2.createMatcher()
            self.event = False
            def proc2_2():
                for m in callAPI(r, "testmodule1", "method3", {'a' : 1, 'b' : 2}):
                    yield m
            def callback_2(event, matcher):
                self.event = event
                if False:
                    yield
            for m in r.withCallback(proc2_2(), callback_2, matcher):
                yield m
            if not self.event:
                for m in r.waitWithTimeout(0.1, matcher):
                    yield m
                if not r.timeout:
                    self.event = r.event
            if self.event:
                apiResults.append((self.event.result, self.event.version))
            else:
                apiResults.append(False)
            with open(os.path.join(basedir, 'testmodule2.py'), 'wb') as f:
                f.write(module2v2)
            for m in s.moduleloader.delegate(s.moduleloader.reloadModules(['tests.gensrc.testmodule2.TestModule2'])):
                yield m
            matcher = ModuleTestEvent2.createMatcher()
            self.event = False
            def proc2_3():
                for m in callAPI(r, "testmodule1", "method3", {'a' : 1, 'b' : 2}):
                    yield m
            def callback_3(event, matcher):
                self.event = event
                if False:
                    yield
            for m in r.withCallback(proc2_3(), callback_3, matcher):
                yield m
            if not self.event:
                for m in r.waitWithTimeout(0.1, matcher):
                    yield m
                if not r.timeout:
                    self.event = r.event
            if self.event:
                apiResults.append((self.event.result, self.event.version))
            else:
                apiResults.append(False)
            with open(os.path.join(basedir, 'testmodule1.py'), 'wb') as f:
                f.write(module1v3)
            with open(os.path.join(basedir, 'testmodule2.py'), 'wb') as f:
                f.write(module2v3)
            for m in s.moduleloader.delegate(s.moduleloader.reloadModules(['tests.gensrc.testmodule1.TestModule1','tests.gensrc.testmodule2.TestModule2'])):
                yield m
            for m in callAPI(r, "testmodule1", "method1", {}):
                yield m
            apiResults.append(r.retvalue)
            matcher = ModuleTestEvent2.createMatcher()
            self.event = False
            def proc2_4():
                for m in callAPI(r, "testmodule1", "method3", {'a' : 1, 'b' : 2}):
                    yield m
            def callback_4(event, matcher):
                self.event = event
                if False:
                    yield
            for m in r.withCallback(proc2_4(), callback_4, matcher):
                yield m
            if not self.event:
                for m in r.waitWithTimeout(0.1, matcher):
                    yield m
                if not r.timeout:
                    self.event = r.event
            if self.event:
                apiResults.append((self.event.result, self.event.version))
            else:
                apiResults.append(False)
            try:
                for m in r.executeWithTimeout(1.0, callAPI(r, "testmodule1", "notexists", {})):
                    yield m
            except ValueError:
                apiResults.append(True)
            except:
                apiResults.append(False)
            else:
                apiResults.append(False)
            for m in s.moduleloader.delegate(s.moduleloader.unloadByPath("tests.gensrc.testmodule1.TestModule1")):
                yield m
        r.main = testproc
        r.start()
        s.serve()
        print(repr(apiResults))
        self.assertEqual(apiResults, ['version1', 3, 'test', (3, 'version1'),
                                      {'method1':'Run method1', 'method2':'Run method2', 'method3':'Run method3', 'method4': 'Run method4', 'discover':'Discover API definitions. Set details=true to show details'},
                                      'version2', (3, 'version1'), (3, 'version2'), 'version3', (3, 'version3'), True])


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 2015/7/30

:author: hubo
'''
from pprint import pprint
from vlcp.server import Server
from vlcp.event import Client, RoutineContainer
from vlcp.protocol.jsonrpc import JsonRPC, JsonRPCConnectionStateEvent
import logging

jsonrpc = JsonRPC()

class MainRoutine(RoutineContainer):
    def main(self):
        connected = JsonRPCConnectionStateEvent.createMatcher(state = JsonRPCConnectionStateEvent.CONNECTION_UP)
        yield (connected,)
        connection = self.event.connection
        for m in jsonrpc.querywithreply('list_dbs', [], connection, self):
            yield m
        if self.jsonrpc_error:
            pprint(self.jsonrpc_error)
        else:
            pprint(self.jsonrpc_result)
        dbname = self.jsonrpc_result[0]
        for m in jsonrpc.querywithreply('get_schema', [dbname], connection, self):
            yield m
        if self.jsonrpc_error:
            pprint(self.jsonrpc_error)
        else:
            pprint(self.jsonrpc_result)
        for m in connection.shutdown():
            yield m

if __name__ == '__main__':
    logging.basicConfig()
    s = Server()
    #s.scheduler.logger.setLevel(logging.DEBUG)
    #JsonRPC.debugging = True
    #JsonRPC._logger.setLevel(logging.DEBUG)
    routine = MainRoutine(s.scheduler)
    routine.start()
    mgt_conn = Client('unix:/var/run/openvswitch/db.sock', jsonrpc, s.scheduler)
    mgt_conn.start()
    s.serve()
    
